// This class provides various utility methods for processing arrays
// of integers

import java.util.*;

public class Arraytools {
    
    private static Random generator = new Random();
    
    // returns a random integer from min to max, inclusive
    public static int randomInt(int min, int max) {
        if (min > max) return 0;
        return min + generator.nextInt(max - min + 1);
    }

    // returns an array of random values in the range min to max
    public static int[] randomArray(int length, int min, int max) {
        int[] data = new int[length];
        for (int i = 0; i < data.length; i++) {
            data[i] = randomInt(min, max);
        }
        return data;
    }
    
    // returns an array of random values in the range 0 to length-1
    public static int[] randomArray(int length) {
        return randomArray(length, 0, length-1);
    }

    // prints all elements of an array
    public static void print(int[] data) {
        printRegion(data, 0, data.length-1);
    }

    // prints elements from data[first] through data[last], inclusive
    public static void printRegion(int[] data, int first, int last) {
        for (int i = first; i <= last; i++) {
            System.out.print(data[i] + " ");
        }
        System.out.println();
    }

    // randomly scrambles an array by swapping elements count times
    public static void scramble(int[] data, int count) {
        for (int i = 0; i < count; i++) {
            int j = randomInt(0, data.length-1);
            int k = randomInt(0, data.length-1);
            swap(data, j, k);
        }
    }

    // randomly scrambles an array
    public static void scramble(int[] data) {
        scramble(data, data.length);
    }

    // reorders data[i] and data[j] so that data[i] <= data[j]
    public static void order(int[] data, int i, int j) {
        if (data[i] > data[j]) {
            swap(data, i, j);
        }
    }

    // returns true if array a1 contains exactly the same elements as a2
    public static boolean hasSameElements(int[] a1, int[] a2) {
        Arrays.sort(a1);
        Arrays.sort(a2);
        return Arrays.equals(a1, a2);
    }

    // returns a sorted array of random values in the range min to max
    public static int[] randomArraySorted(int length, int min, int max) {
        int[] data = randomArray(length, min, max);
        Arrays.sort(data);
        return data;
    }

    // returns a sorted array of random values in the range 0 to length-1
    public static int[] randomArraySorted(int length) {
        return randomArraySorted(length, 0, length-1);
    }

    // returns an array of random values in reverse sorted order
    public static int[] randomArrayReverseSorted(int length, int min, int max) {
        int[] data = randomArray(length, min, max);
        Arrays.sort(data);
        for (int i = 0; i < data.length/2; i++) {
            swap(data, i, data.length-1-i);
        }
        return data;
    }

    // returns a random array that is partially sorted. fraction = 0
    // means completely unsorted. fraction = 1 means completely sorted.
    public static int[] randomArrayPartiallySorted(int length, int min, int max,
                                                   double fraction) {
        int[] data = randomArray(length, min, max);
        Arrays.sort(data);
        int numSwaps = (int) ((1 - fraction) * data.length);
        scramble(data, numSwaps);
        return data;
    }

    // returns a random array with the given fraction all the same element
    public static int[] randomArrayWithSameElement(int length, int min, int max,
                                                   double fraction) {
        int[] data = new int[length];
        int j = (int) (fraction * data.length);
        int r = randomInt(min, max);
        for (int i = 0; i < j; i++) {
            data[i] = r;
        }
        for (int i = j; i < data.length; i++) {
            data[i] = randomInt(min, max);
        }
        scramble(data);
        return data;
    }

    // returns an array of int values entered from the keyboard
    // on a single line, with each value separated by whitespace
    public static int[] inputArray() {
        Scanner keyboard = new Scanner(System.in);
        String input = keyboard.nextLine();
        String[] strings = input.split("\\s"); // split on whitespace
        int[] data = new int[strings.length];
        for (int i = 0; i < strings.length; i++) {
            data[i] = Integer.parseInt(strings[i]);
        }
        return data;
    }

    //----------------------------------------------------------------------
    // implemented in class

    // swaps data[i] and data[j]
    public static void swap(int[] data, int i, int j) {
        int temp = data[i];
        data[i] = data[j];
        data[j] = temp;
    }
    
    // returns true if an array is sorted or false otherwise
    public static boolean isSorted(int[] data) {
        for (int i = 0; i < data.length-1; i++) {
            if (data[i] > data[i+1]) {
                return false;
            }
        }
        return true;
    }

    // copies source[first...last] to dest array, beginning at position k of dest
    public static void copy(int[] source, int first, int last, int[] dest, int k) {
        for (int i = first; i <= last; i++) {
            dest[k] = source[i];
            k++;
        }
    }

    // returns a new copy of an array
    public static int[] clone(int[] data) {
        int[] data2 = new int[data.length];
        copy(data, 0, data.length-1, data2, 0);
        return data2;
    }

    //----------------------------------------------------------------------
    // test program

    public static void main(String[] args) {

        int[] values = randomArray(20, 0, 200);
        //int[] values = randomArray(20, 80, 99);
        //int[] values = randomArray(20);
        //int[] values = randomArraySorted(20, 0, 200);
        //int[] values = randomArrayReverseSorted(20, 0, 200);
        //int[] values = randomArrayPartiallySorted(20, 0, 20, 0.80);
        //int[] values = randomArrayWithSameElement(20, 0, 20, 0.80);
        //int[] values = inputArray();

        print(values);
        //printRegion(values, 2, 5);
        //scramble(values);
        //print(values);

        /*
        // code for testing isSorted method
        int[] values = randomArray(10);
        print(values);
        System.out.printf("isSorted says %s\n\n", isSorted(values));
        values = randomArraySorted(10);
        print(values);
        System.out.printf("isSorted says %s\n\n", isSorted(values));
        values = randomArrayPartiallySorted(10, 0, 9, 0.8);
        print(values);
        System.out.printf("isSorted says %s\n\n", isSorted(values));
        */

        /*
        // code for testing copy method
        int[] source = randomArray(10, 1, 9);
        int[] dest = randomArrayWithSameElement(15, 0, 0, 1.0);
        System.out.print("source: ");
        print(source);
        System.out.print("  dest: ");
        print(dest);
        System.out.println("copying source[1..8] to dest starting at dest[3]");
        copy(source, 1, 8, dest, 3);
        System.out.print("  dest: ");
        print(dest);
        */

        /*
        // code for testing clone method
        int[] a1 = randomArray(10);
        print(a1);
        int[] a2 = clone(a1);
        if (a2 == null) {
            System.out.println("ERROR: clone returned null");
        } else if (a1 == a2) {
            System.out.println("ERROR: clone returned the same array object");
        } else if (!Arrays.equals(a1, a2)) {
            print(a2);
            System.out.println("ERROR: arrays are different");
        } else {
            print(a2);
            System.out.println("Arrays are identical");
        }
        */
    }
}
